#!/usr/bin/env python
# wordpos.py
#
# Frequencies of word positions in a line
# Written by Elias Schwerdtfeger, http://voynich.tamagothi.de
#
# This script uses some features of the MySQL-RDBMS, it is probably
# not portable to any other RDBMS. But you may try it, if you like.
# The temporary table is much faster than a subselect.
#
# $Id: wordpos.py,v 1.1 2008/04/06 15:29:01 elias Exp $
#
# $Log: wordpos.py,v $
# Revision 1.1  2008/04/06 15:29:01  elias
# Initial revision
#

import MySQLdb

# INIT PART
# ---------

use_trans = 'H'
from_start = True

# Additional conditions for selecting a part of the manuscript
where_and = "AND 1=1 "

# This is the default from_clause for all queries
from_clause = """
FROM voy_word
JOIN voy_lineword ON lword_word = word_id
JOIN voy_line ON line_id = lword_line
JOIN voy_page ON page_id = line_page
"""

# PARAMETER PARSING PART
# ----------------------

import getopt
import sys

optlist, args = getopt.getopt(sys.argv[1:], 't:l:i:rh')
for opt, par in optlist:
    if opt == '-t':
        use_trans = par
    elif opt == '-l':
        pp = ["'%s'" % (p, ) for p in par]
        par = ', '.join(pp)
        where_and += 'AND currier_lang IN (%s) ' % (par)
    elif opt == '-i':
        pp = ["'%s'" % (p, ) for p in par]
        par = ', '.join(pp)
        where_and += 'AND illustration_type IN (%s) ' % (par)
    elif opt == '-r':
        from_start = False
    elif opt == '-h':
        print 'Get frequencies for word position in a line'
        print
        print 'Options'
        print '-t<code>  Select transcription by code'
        print '-l<lang>  Select currier languages'
        print '-i<code>  Select illustration types'
        print '-r        Create counts for the line endings'
        print '-h        Show this help'
        sys.exit()
        
# This is the default where-clause for all queries
where_clause = """
WHERE line_trans = '%s'
AND readable = 'Y'
%s
""" % (use_trans, where_and)

# PROCESSING PART
# ---------------

# Change your login settings here
# The user must be allowed to create a temporary table.
con = MySQLdb.connect(host='localhost',
                      db='voynich',
                      user='voynich',
                      passwd='voynich')

cur = con.cursor()

# Create temporary tables for the word frequencies in the
# selected part of the manuscript to avoid a subselect.
cur.execute("""
CREATE TEMPORARY TABLE voy_wfreq
(
   wfreq_word SMALLINT(6) PRIMARY KEY,
   total SMALLINT(6)
)""")
cur.execute("""
INSERT INTO voy_wfreq (wfreq_word, total)
SELECT word_id, COUNT(*) AS total
""" + from_clause + where_clause + """
GROUP BY word_id
""")

# Now extend the from-clause to include the total counts
# from the temporary table.
from_clause += """\
JOIN voy_wfreq ON wfreq_word = word_id
"""

# Execute the SQL stmt for the desired case.
# This could be done more elegant, but less readable...
if from_start:
    cur.execute("""
    SELECT eva, position, COUNT(*),
    100.0 * COUNT(*) / total, total
    """ + from_clause + where_clause + """
    GROUP BY eva, position
    HAVING total > 1
    """)
else:
    cur.execute("""
    SELECT eva, wordcount - position + 1 AS pt, COUNT(*),
    100.0 * COUNT(*) / total, total
    """ + from_clause + where_clause + """
    GROUP BY eva, pt
    HAVING total > 1
    """)
lasteva = ''
for l in cur.fetchall():
    if l[0] != lasteva:
        print
        print '%s (%d)' % (l[0], l[4]),
        lasteva = l[0]
    # Want decimal places? Change the format string!
    print '\t%d (%d %.0f%%)' % (l[1], l[2], l[3]), 
# End file with a newline character.
print

# Clean up the temporary table
cur.execute("DROP TABLE voy_wfreq")

